/*
 * Copyright (c) 2006, WSO2 Inc. (http://www.wso2.org) All Rights Reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.wso2.registry.jdbc.mediatypes;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.wso2.registry.Artifact;
import org.wso2.registry.RegistryConstants;
import org.wso2.registry.RegistryException;
import org.wso2.registry.i18n.Messages;
import org.wso2.registry.jdbc.ConnectionFactory;
import org.wso2.registry.jdbc.JDBCRegistry;
import org.wso2.registry.jdbc.dao.VersionedArtifactDAO;
import org.wso2.registry.jdbc.mediatypes.builtin.DefaultMediaTypeHandler;
import org.wso2.registry.jdbc.mediatypes.builtin.SQLQueryMediaTypeHandler;
import org.wso2.registry.jdbc.mediatypes.builtin.SynapseRepositoryMediaTypeHandler;
import org.wso2.usermanager.Realm;

import java.sql.Connection;
import java.sql.SQLException;
import java.util.HashMap;
import java.util.Map;

/**
 * Handles the actions by delegating them to registered media type handlers. If a mediatype handler
 * is not registered for a media type, its handling will be delegated to the
 * DefaultMediaTypeHandler. This class initializes all media type handlers at start up so that they
 * can be looked up by the media type string.
 */
public class MediaTypeManager {

    private static final Log log = LogFactory.getLog(JDBCRegistry.class);

    private ConnectionFactory connectionFactory;

    private VersionedArtifactDAO artifactDAO = new VersionedArtifactDAO();

    private Map mediaTypeHandlers = new HashMap();
    private DefaultMediaTypeHandler defaultMediaTypeHandler;

    public MediaTypeManager(ConnectionFactory connectionFactory, Realm realm) {
        this.connectionFactory = connectionFactory;

        defaultMediaTypeHandler = new DefaultMediaTypeHandler(connectionFactory, realm, this);
        mediaTypeHandlers.put(RegistryConstants.DEFAULT_MEDIA_TYPE, defaultMediaTypeHandler);

        MediaTypeHandler sqlMediaTypeHandler =
                new SQLQueryMediaTypeHandler(connectionFactory, realm, this);
        mediaTypeHandlers.put(RegistryConstants.SQL_QUERY_MEDIA_TYPE, sqlMediaTypeHandler);

        MediaTypeHandler synRepoMediaTypeHandler =
                new SynapseRepositoryMediaTypeHandler(connectionFactory, realm, this);
        mediaTypeHandlers.put(
                RegistryConstants.SYNPASE_REPOSITORY_MEDIA_TYPE, synRepoMediaTypeHandler);
    }

    public Artifact get(String path) throws RegistryException {        

        Artifact artifact = defaultMediaTypeHandler.get(path, null);
        
        Artifact processedArtifact = null;

        if (artifact == null) {
            throw new RegistryException("No resource found at path " + path);
        }

        if (artifact.getMediaType() != null) {
            MediaTypeHandler mediaTypeHandler =
                    (MediaTypeHandler) mediaTypeHandlers.get(artifact.getMediaType());

            if (mediaTypeHandler != null) {
                processedArtifact = mediaTypeHandler.get(path, artifact);
                if (processedArtifact == null) {
                    processedArtifact = defaultMediaTypeHandler.get(path, artifact);
                }
            } else {
                processedArtifact = defaultMediaTypeHandler.get(path, artifact);
            }
        } else {
            processedArtifact = defaultMediaTypeHandler.get(path, artifact);
        }

        if (processedArtifact != null) {
            return processedArtifact;
        } else {
            return artifact;
        }
    }

    public void put(String path, Artifact artifact) throws RegistryException {

        if (artifact.getMediaType() != null) {
            MediaTypeHandler mediaTypeHandler =
                    (MediaTypeHandler) mediaTypeHandlers.get(artifact.getMediaType());

            if (mediaTypeHandler != null) {
                if (!mediaTypeHandler.put(path, artifact)) {
                    defaultMediaTypeHandler.put(path, artifact);
                }
            } else {
                defaultMediaTypeHandler.put(path, artifact);
            }
        } else {
            defaultMediaTypeHandler.put(path, artifact);
        }
    }

    public void delete(String path) throws RegistryException {

        Artifact artifact = getRawArtifact(path);

        if (artifact == null) {
            throw new RegistryException("Could not delete a non-existing resource.");
        }

        if (artifact.getMediaType() != null) {
            MediaTypeHandler mediaTypeHandler =
                    (MediaTypeHandler) mediaTypeHandlers.get(artifact.getMediaType());

            if (mediaTypeHandler != null) {
                if (!mediaTypeHandler.delete(path)) {
                    defaultMediaTypeHandler.delete(path);
                }
            } else {
                defaultMediaTypeHandler.delete(path);
            }
        } else {
            defaultMediaTypeHandler.delete(path);
        }
    }

    public MediaTypeHandler getMediaTypeHandler(String mediaType) {
        return (MediaTypeHandler) mediaTypeHandlers.get(mediaType);
    }

    public MediaTypeHandler getDefaultMediaTypeHandler() {
        return defaultMediaTypeHandler;
    }

    private Artifact getRawArtifact(String path) throws RegistryException {

        Artifact artifact;

        Connection conn = connectionFactory.getConnection();
        try {
            artifact = artifactDAO.getLatestVersion(path, conn);

        } catch (SQLException e) {

            String msg = Messages.getMessage("resource.get.fail", path);
            log.error(msg, e);
            throw new RegistryException(msg);

        } finally {
            try {
                conn.close();
            } catch (SQLException e) {
                e.printStackTrace();
            }
        }

        return artifact;
    }
}
